@@ -5,7 +5,7 @@ from lxml.etree import Element
 from rdflib import Graph, Literal, URIRef, XSD, RDF, RDFS, OWL
 from slugify import slugify
 from acdh_tei_pyutils.utils import make_entity_label
-from acdh_cidoc_pyutils.namespaces import CIDOC, NSMAP
+from acdh_cidoc_pyutils.namespaces import CIDOC, FRBROO, NSMAP, DATE_ATTRIBUTE_DICT
 
 
 def normalize_string(string: str) -> str:
@@ -45,19 +45,41 @@ def coordinates_to_p168(
     return g
 
 
-def extract_begin_end(date_object: Union[Element, dict]) -> tuple[str, str]:
-    begin, end = "", ""
-    if date_object.get("when-iso", "") != "":
-        return (date_object.get("when-iso"), date_object.get("when-iso"))
-    elif date_object.get("when", "") != "":
-        return (date_object.get("when"), date_object.get("when"))
-    begin = date_object.get("notBefore", "")
-    end = date_object.get("notAfter", "")
-    if begin != "" and end == "":
-        end = begin
-    if end != "" and begin == "":
-        begin = end
-    return (begin, end)
+def extract_begin_end(
+    date_object: Union[Element, dict],
+    fill_missing=True,
+    attribute_map=DATE_ATTRIBUTE_DICT,
+) -> tuple[Union[str, bool], Union[str, bool]]:
+    final_start, final_end = None, None
+    start, end, when = None, None, None
+    for key, value in attribute_map.items():
+        date_value = date_object.get(key)
+        if date_value and value == "start":
+            start = date_value
+        if date_value and value == "end":
+            end = date_value
+        if date_value and value == "when":
+            when = date_value
+    if fill_missing:
+        if start or end or when:
+            if start and end:
+                final_start, final_end = start, end
+            elif start and not end and not when:
+                final_start, final_end = start, start
+            elif end and not start and not when:
+                final_start, final_end = end, end
+            elif when and not start and not end:
+                final_start, final_end = when, when
+    else:
+        if start and end:
+            final_start, final_end = start, end
+        elif start and not end and not when:
+            final_start, final_end = start, None
+        elif end and not start and not when:
+            final_start, final_end = None, end
+        elif when and not start and not end:
+            final_start, final_end = when, when
+    return final_start, final_end
 
 
 def date_to_literal(date_str: str) -> Literal:
@@ -235,3 +257,104 @@ def make_ed42_identifiers(
                     )
                 )
     return g
+
+
+def make_occupations(subj: URIRef, node: Element, domain: str, prefix="occupation", id_xpath=False, default_lang="de"):
+    g = Graph()
+    occ_uris = []
+    base_uri = f"{subj}/{prefix}"
+    for i, x in enumerate(node.xpath('.//tei:occupation', namespaces=NSMAP)):
+        try:
+            lang = x.attrib["{http://www.w3.org/XML/1998/namespace}lang"]
+        except KeyError:
+            lang = default_lang
+        occ_text = normalize_string(" ".join(x.xpath('.//text()')))
+        occ_id = f"{i}"
+        if id_xpath:
+            try:
+                occ_id = x.xpath(id_xpath, namespaces=NSMAP)[0]
+            except IndexError:
+                pass
+        else:
+            occ_id = occ_id
+        if occ_id.startswith('#'):
+            occ_id = occ_id[1:]
+        occ_uri = URIRef(f"{base_uri}/{occ_id}")
+        occ_uris.append(occ_uri)
+        g.add((occ_uri, RDF.type, FRBROO["F51"]))
+        g.add((occ_uri, RDFS.label, Literal(occ_text, lang=lang)))
+        g.add((
+            subj, CIDOC["P14i_performed"], occ_uri
+        ))
+        begin, end = extract_begin_end(x)
+        if begin or end:
+            ts_uri = URIRef(f"{occ_uri}/timestamp")
+            g.add((occ_uri, CIDOC["P4_has_time-span"], ts_uri))
+            g += create_e52(ts_uri, begin_of_begin=begin, end_of_end=end)
+    return (g, occ_uris)
+
+
+def make_birth_death_entities(
+    subj: URIRef,
+    node: Element,
+    domain: str,
+    event_type="birth",
+    verbose=False,
+    default_prefix="Geburt von",
+    default_lang="de",
+    date_node_xpath="",
+    place_id_xpath="//tei:placeName/@key",
+):
+    g = Graph()
+    name_node = node.xpath(".//tei:persName[1]", namespaces=NSMAP)[0]
+    label, label_lang = make_entity_label(name_node, default_lang=default_lang)
+    if event_type not in ["birth", "death"]:
+        return (g, None, None)
+    if event_type == "birth":
+        cidoc_property = CIDOC["P98_brought_into_life"]
+        cidoc_class = CIDOC["E67_Birth"]
+    else:
+        cidoc_property = CIDOC["P100_was_death_of"]
+        cidoc_class = CIDOC["E69_Death"]
+    xpath_expr = f".//tei:{event_type}[1]"
+    place_xpath = f"{xpath_expr}{place_id_xpath}"
+    if date_node_xpath != "":
+        date_xpath = f"{xpath_expr}/{date_node_xpath}"
+    else:
+        date_xpath = xpath_expr
+    try:
+        node.xpath(xpath_expr, namespaces=NSMAP)[0]
+    except IndexError as e:
+        if verbose:
+            print(subj, e)
+            return (g, None, None)
+    event_uri = URIRef(f"{subj}/{event_type}")
+    time_stamp_uri = URIRef(f"{event_uri}/timestamp")
+    g.set((event_uri, cidoc_property, subj))
+    g.set((event_uri, RDF.type, cidoc_class))
+    g.add(
+        (event_uri, RDFS.label, Literal(f"{default_prefix} {label}", lang=label_lang))
+    )
+    g.set((event_uri, CIDOC["P4_has_time-span"], time_stamp_uri))
+    try:
+        date_node = node.xpath(date_xpath, namespaces=NSMAP)[0]
+        process_date = True
+    except IndexError:
+        process_date = False
+    if process_date:
+        start, end = extract_begin_end(date_node)
+        try:
+            g += create_e52(time_stamp_uri, begin_of_begin=start, end_of_end=end)
+        except TypeError:
+            pass
+    try:
+        place_node = node.xpath(place_xpath, namespaces=NSMAP)[0]
+        process_place = True
+    except IndexError:
+        process_place = False
+    if process_place:
+        if place_node.startswith("#"):
+            place_node = place_node[1:]
+        place_uri = URIRef(f"{domain}{place_node}")
+        g.add((event_uri, CIDOC["P7_took_place_at"], place_uri))
+    return (g, event_uri, time_stamp_uri)
